//---------------------------------------------------
// Input Correction Function
//---------------------------------------------------

#include "Module_Definitions.h"

__m128 one_CB = _mm_set_ps1(1.0f);
__m128 zero_CB = _mm_set_ps1(0.0f);
__m128 compare_offset = _mm_set_ps1(0xFFFFFFFF);
__m128 Lum_Shift_sse_CB = _mm_set_ps1(Lum_Shift_CB);
__m128 Lum_upper_range_CB = _mm_set_ps1(float(2*(235-16) + 16));
__m128 Lum_lower_range_CB = _mm_set_ps1(float(-2*(235-16) + 16));

__m128 satshift_sse = _mm_set_ps1(satshift);
__m128 hue_CB_see0 = _mm_set_ps1(hue_CB_mx[0]);
__m128 hue_CB_see1 = _mm_set_ps1(hue_CB_mx[1]);
__m128 hue_CB_see2 = _mm_set_ps1(hue_CB_mx[2]);
__m128 hue_CB_see3 = _mm_set_ps1(hue_CB_mx[3]);

void colour_balance(__m128* R, __m128* G, __m128* B, __m128* X, __m128* Y, __m128* L)
{

	// Convert to XYL
	rgb_to_xyl(R, G, B, X, Y, L);

	// Luminance Correction
	*L = _mm_add_ps(*L, Lum_Shift_sse_CB);
	*L = _mm_max_ps(*L, Lum_lower_range_CB);
	*L = _mm_min_ps(*L, Lum_upper_range_CB);
	__m128 orgLum = *L;

	// XY Correction
	__m128 dist, new_gain, comp1, comp0, m0, m1, m2, m3, m4, m5;

	// Calculate the proposed saturation
	m0 = _mm_mul_ps(*X, *X);
	m1 = _mm_mul_ps(*Y, *Y);
	dist = _mm_add_ps(m0, m1);
	dist = _mm_sqrt_ps(dist);
	new_gain = _mm_add_ps(dist, satshift_sse);
	
	// Compare with one_CB and zero_CB - make 1 if true else 0
	comp1 = _mm_cmpgt_ps(new_gain, one_CB);
	comp1 = _mm_div_ps(comp1, compare_offset);
	comp0 = _mm_cmplt_ps(new_gain, zero_CB);
	comp0 = _mm_div_ps(comp0, compare_offset);
	
	// Create 1 - compX 
	m1 = _mm_sub_ps(one_CB, comp1);
	m0 = _mm_sub_ps(one_CB, comp0);

	// If > 1 then set 1-dist
	m3 = _mm_sub_ps(one_CB, dist);
	m3 = _mm_mul_ps(comp1, m3);  // greater than

	// If < 1 then...
	// If < 0 = -dist
	m5 = _mm_sub_ps(zero_CB,dist);
	m4 = _mm_mul_ps(comp0, m5);
	// else = satgain_sse
	m5 = _mm_mul_ps(m0, satshift_sse);

	// Add possibities
	m5 = _mm_add_ps(m4,m5);
	m5 = _mm_mul_ps(m1,m5);
	new_gain = _mm_add_ps(m5, m3);
	new_gain = _mm_div_ps(new_gain, dist);
	new_gain = _mm_add_ps(new_gain, one_CB);

	m0 = _mm_mul_ps(hue_CB_see0, *X);
	m1 = _mm_mul_ps(hue_CB_see1, *Y);
		
	m2 = _mm_mul_ps(hue_CB_see2, *X);
	m3 = _mm_mul_ps(hue_CB_see3, *Y);
	
	*X = _mm_add_ps(m0, m1);
	*X = _mm_mul_ps(*X, new_gain);
	*Y = _mm_add_ps(m2, m3);
	*Y = _mm_mul_ps(*Y, new_gain);

	// Convert back to RGB space
	__m128 Rn, Gn, Bn;
	
	xyl_to_rgb(&Rn, &Gn, &Bn, X, Y, L);
	
	float* org_Lumf = (float*)L;
	float* Rf = (float*)R;
	float* Gf = (float*)G;
	float* Bf = (float*)B;
	float* Rnf = (float*)&Rn;
	float* Gnf = (float*)&Gn;
	float* Bnf = (float*)&Bn;

	for(int i=0; i<4; i++)
	{
		if (area_selection == 1)
		{
			if (*(org_Lumf+i) >= TH1)
			{
				*(Rf+i) = *(Rnf+i);
				*(Gf+i) = *(Gnf+i);
				*(Bf+i) = *(Bnf+i);
			}
		}
		else if (area_selection == 2)
		{
			if (*(org_Lumf+i) > TH1 && *(org_Lumf+i) < TH2)
			{
				*(Rf+i) = *(Rnf+i);
				*(Gf+i) = *(Gnf+i);
				*(Bf+i) = *(Bnf+i);
			}
		}
		else if (area_selection == 3)
		{
			if (*(org_Lumf+i) <= TH1)
			{
				*(Rf+i) = *(Rnf+i);
				*(Gf+i) = *(Gnf+i);
				*(Bf+i) = *(Bnf+i);
			}
		}
	}
	R = (__m128*)Rf;
	G = (__m128*)Gf;
	B = (__m128*)Bf;

}