//---------------------------------------------------
// Histogram Equalisation Function
//---------------------------------------------------

#include "Module_Definitions.h"

__m128 zero_he = _mm_set_ps1(0.0f);
__m128 val_255 = _mm_set_ps1(255.0f);
__m128 obl_sse = _mm_set_ps1(outputblacklevel);
__m128 owl_sse = _mm_set_ps1(outputwhitelevel);
__m128 BW_DIFF_sse = _mm_set_ps1(BW_DIFF);
__m128 Black_Level_sse = _mm_set_ps1(Black_Level);

void histogram_equalisation(__m128* R, __m128* G, __m128* B, float* power_lut)
{
	
	__m128 R_org = *R;
	__m128 G_org = *G;
	__m128 B_org = *B;

	*R = _mm_sub_ps(*R, Black_Level_sse);
	*G = _mm_sub_ps(*G, Black_Level_sse);
	*B = _mm_sub_ps(*B, Black_Level_sse);

	*R = _mm_mul_ps(*R, BW_DIFF_sse);
	*G = _mm_mul_ps(*G, BW_DIFF_sse);
	*B = _mm_mul_ps(*B, BW_DIFF_sse);

	*R = _mm_min_ps(val_255, *R);
	*R = _mm_max_ps(zero_he, *R);
	*G = _mm_min_ps(val_255, *G);
	*G = _mm_max_ps(zero_he, *G);
	*B = _mm_min_ps(val_255, *B);
	*B = _mm_max_ps(zero_he, *B);
	
	// pow lookup - requires mode change
	float* Rf = (float*)R;
	float* Gf = (float*)G;
	float* Bf = (float*)B;
	for (int i=0; i<4; i++)
	{
		*(Rf+i) = *(power_lut+int(*(Rf+i)));
		*(Gf+i) = *(power_lut+int(*(Gf+i)));
		*(Bf+i) = *(power_lut+int(*(Bf+i)));
	}
	R = (__m128*)Rf;
	G = (__m128*)Gf;
	B = (__m128*)Bf;

	// range limit output
	*R = _mm_min_ps(owl_sse, *R);
	*R = _mm_max_ps(obl_sse, *R);
	*G = _mm_min_ps(owl_sse, *G);
	*G = _mm_max_ps(obl_sse, *G);
	*B = _mm_min_ps(owl_sse, *B);
	*B = _mm_max_ps(obl_sse, *B);

	//Conditional modification
	if (channel_selection == 1.5)
	{	*G = G_org;
		*R = R_org;
	}
	else if (channel_selection == 2.5)
	{	*R = R_org;
		*B = B_org;
	}
	else if (channel_selection == 3.5)
	{	*G = G_org;
		*B = B_org;
	}
}