//---------------------------------------------------
// Input Correction Function
//---------------------------------------------------

#include "Module_Definitions.h"

__m128 one_IC = _mm_set_ps1(1.0f);
__m128 Lum_Shift_sse = _mm_set_ps1(Lum_Shift_IC);
__m128 Lum_upper_range = _mm_set_ps1(float(2*(235-16) + 16));
__m128 Lum_lower_range = _mm_set_ps1(float(-2*(235-16) + 16));

__m128 Lum_Gain_sse = _mm_set_ps1(LumGain);
__m128 satgain_sse = _mm_set_ps1(satgain);
__m128 hue_IC_see0 = _mm_set_ps1(hue_IC_mx[0]); 
__m128 hue_IC_see1 = _mm_set_ps1(hue_IC_mx[1]);
__m128 hue_IC_see2 = _mm_set_ps1(hue_IC_mx[2]);
__m128 hue_IC_see3 = _mm_set_ps1(hue_IC_mx[3]);

void input_correction(__m128* R, __m128* G, __m128* B, __m128* X, __m128* Y, __m128* L)
{
	// Convert to XYL

	rgb_to_xyl(R, G, B, X, Y, L);

	// Luminance Correction
	*L = _mm_add_ps(*L, Lum_Shift_sse);
	*L = _mm_max_ps(*L, Lum_lower_range);
	*L = _mm_min_ps(*L, Lum_upper_range);

	*L = _mm_mul_ps(*L, Lum_Gain_sse);
	*L = _mm_max_ps(*L, Lum_lower_range);
	*L = _mm_min_ps(*L, Lum_upper_range);

	// XY Correction
	__m128 dist, x_sqr, y_sqr, new_gain, comp, m0, m1, m2, m3;

	//...saturation calculation
	x_sqr = _mm_mul_ps(*X, *X);
	y_sqr = _mm_mul_ps(*Y, *Y);
	m1 = _mm_add_ps(x_sqr, y_sqr);
	dist = _mm_sqrt_ps(m1);

	// ...saturation post-modification
	new_gain = _mm_mul_ps(dist, satgain_sse);

	// ...modify if too high
	comp = _mm_cmpgt_ps(new_gain, one_IC);
	comp = _mm_div_ps(comp, _mm_set_ps1(0xFFFFFFFF));
	m1 = _mm_sub_ps(one_IC, comp);
	dist = _mm_rcp_ps(dist);
	m0 = _mm_mul_ps(m1, satgain_sse);
	m1 = _mm_mul_ps(comp, dist);
	new_gain = _mm_add_ps(m0, m1);
	
	// ... matrix multiply and *constant new_gain
	m0 = _mm_mul_ps(hue_IC_see0, *X);
	m1 = _mm_mul_ps(hue_IC_see1, *Y);
	m2 = _mm_mul_ps(hue_IC_see2, *X);
	m3 = _mm_mul_ps(hue_IC_see3, *Y);
	*X = _mm_add_ps(m0, m1);
	*X = _mm_mul_ps(*X, new_gain);
	*Y = _mm_add_ps(m2, m3);
	*Y = _mm_mul_ps(*Y, new_gain);

	// Convert back to RGB space
	xyl_to_rgb(R, G, B, X, Y, L);

}
