#include "Module_Definitions.h"

// Matrix + Vector Parameters RGB to XYL
__m128 rx0_0 = _mm_set_ps1(0.257f);
__m128 rx0_1 = _mm_set_ps1(0.504f);
__m128 rx0_2 = _mm_set_ps1(0.098f);

__m128 rx1_0 = _mm_set_ps1(-0.148f);
__m128 rx1_1 = _mm_set_ps1(-0.291f);
__m128 rx1_2 = _mm_set_ps1(0.439f);

__m128 rx2_0 = _mm_set_ps1(0.439f);
__m128 rx2_1 = _mm_set_ps1(-0.368f);
__m128 rx2_2 = _mm_set_ps1(-0.071f);

// Some useful constant values
__m128 v0 = _mm_set_ps1(0.0f);
__m128 v16 = _mm_set_ps1(16.0f);
__m128 v235 = _mm_set_ps1(235.0f);
__m128 v240 = _mm_set_ps1(240.0f);
__m128 v128 = _mm_set_ps1(128.0f);
__m128 v112 = _mm_set_ps1(112.0f);
__m128 v255 = _mm_set_ps1(255.0f);

// Matrix + Vector Parameters XYL to RGB
__m128 xr0_1 = _mm_set_ps1(1.596f);
__m128 xr0_2 = _mm_set_ps1(1.164f);

__m128 xr1_0 = _mm_set_ps1(-0.391f);
__m128 xr1_1 = _mm_set_ps1(-0.813f);
__m128 xr1_2 = _mm_set_ps1(1.164f);

__m128 xr2_0 = _mm_set_ps1(2.018f);
__m128 xr2_2 = _mm_set_ps1(1.164f);

// RGB to XYL function
void rgb_to_xyl(__m128* R, __m128* G, __m128* B, __m128* X, __m128* Y, __m128* L)
{
	__m128 m00, m01, m02; 

	m00 = _mm_mul_ps(*R, rx0_0);
	m01 = _mm_mul_ps(*G, rx0_1);
	m02 = _mm_mul_ps(*B, rx0_2);
	m00 = _mm_add_ps(m00, m01);
	m02 = _mm_add_ps(m02, v16);
	m00 = _mm_add_ps(m00, m02);
	m00 = _mm_max_ps(m00, v16);
	*L = _mm_min_ps(m00, v235);
	
	__m128 m10, m11, m12;

	m10 = _mm_mul_ps(*R, rx1_0);
	m11 = _mm_mul_ps(*G, rx1_1);
	m12 = _mm_mul_ps(*B, rx1_2);
	m10 = _mm_add_ps(m10, m11);
	m12 = _mm_add_ps(m12, v128);
	m10 = _mm_add_ps(m10, m12);
	m10 = _mm_max_ps(m10, v16);
	m10 = _mm_min_ps(m10, v240);
	m10 = _mm_sub_ps(m10, v128);
	*X = _mm_div_ps(m10, v112);
	
	__m128 m20, m21, m22;

	m20 = _mm_mul_ps(*R, rx2_0);
	m21 = _mm_mul_ps(*G, rx2_1);
	m22 = _mm_mul_ps(*B, rx2_2);
	m20 = _mm_add_ps(m20, m21);
	m22 = _mm_add_ps(m22, v128);
	m20 = _mm_add_ps(m20, m22);
	m20 = _mm_max_ps(m20, v16);
	m20 = _mm_min_ps(m20, v240);
	m20 = _mm_sub_ps(m20, v128);
	*Y = _mm_div_ps(m20, v112);
}

// XYL to RGB function
void xyl_to_rgb(__m128* R, __m128* G, __m128* B, __m128* X, __m128* Y, __m128* L)
{
	*X = _mm_mul_ps(*X, v112);
	*Y = _mm_mul_ps(*Y, v112);
	*L = _mm_sub_ps(*L, v16);

	__m128 a01, a02, a10, a11, a12, a20, a22;

	a01 = _mm_mul_ps(*Y, xr0_1);
	a02 = _mm_mul_ps(*L, xr0_2);
	a01 = _mm_add_ps(a01, a02);
	a01 = _mm_max_ps(a01, v0);
	*R = _mm_min_ps(a01, v255);
	
	a10 = _mm_mul_ps(*X, xr1_0);
	a11 = _mm_mul_ps(*Y, xr1_1);
	a12 = _mm_mul_ps(*L, xr1_2);
	a10 = _mm_add_ps(a10, a11);
	a10 = _mm_add_ps(a10, a12);
	a10 = _mm_max_ps(a10, v0);
	*G = _mm_min_ps(a10, v255);
	
	a20 = _mm_mul_ps(*X, xr2_0);
	a22 = _mm_mul_ps(*L, xr2_2);
	a20 = _mm_add_ps(a20, a22);
	a20 = _mm_max_ps(a20, v0);
	*B = _mm_min_ps(a20, v255);
}