-------------------------------------------------------
-- Created by Ben Cope of Imperial College (May 2005)
-- Luminance Update Block for Primary Colour Correction
-------------------------------------------------------

LIBRARY IEEE;
USE IEEE.std_logic_1164.ALL;
USE IEEE.std_logic_arith.ALL;

LIBRARY UNISIM;
USE UNISIM.vcomponents.ALL;

LIBRARY work;
USE work.Constants.ALL;

ENTITY CB_Lum_alter IS
 PORT(
  -- Input Signals
  clock : IN STD_LOGIC;
  reset: IN STD_LOGIC;
  lum_shift : IN Vector12;
  lum_in : IN Vector18;

  -- Output Signals
  Lum_out : OUT Vector18);
END CB_Lum_alter;

ARCHITECTURE inside OF CB_Lum_alter IS
SIGNAL Lum_2 : Vector18;
SIGNAL Lum_1 : Vector19;
SIGNAL lum_shift_int : Vector19;
SIGNAL Lum_delay : Lum_delay_Array;
SIGNAL Lum_extended : Vector19;

CONSTANT lower_limit : Vector19 := "0000010000000000000"; -- 16/256 with 2I.17F
CONSTANT upper_limit : Vector19 := "0011101011000000000"; -- 235/256 with 2I.17F

BEGIN

 -- Main Process
P1 : PROCESS

BEGIN

 WAIT UNTIL Rising_Edge(clock);

 IF (reset = '1') THEN
  
  Lum_out <= (OTHERS => '0');

 ELSE

 lum_shift_int <= lum_shift(11) & lum_shift & "000000";

  -- LEVEL 1 PIPELINE
  
  -- Apply luminance shift
  Lum_extended <= Lum_in(17) & Lum_in;

  Lum_1 <= signed(Lum_extended) + signed(lum_shift_int);  

  -- LEVEL 2 PIPELINE

  -- Range Limit
  IF (signed(Lum_1) < signed(lower_limit)) THEN
    Lum_2 <= lower_limit(17 DOWNTO 0);
  ELSIF (signed(Lum_1) > signed(upper_limit)) THEN
    Lum_2 <= upper_limit(17 DOWNTO 0);
  ELSE
    Lum_2 <= Lum_1(17 DOWNTO 0);
  END IF;

  -- LEVEL 3 PIPELINE
  lum_delay(0) <= Lum_2;

  -- DELAY Lum by 18 cycles
  FOR i IN 0 TO 17 LOOP
   Lum_delay(i+1) <= Lum_delay(i);
  END LOOP;

  Lum_out <= Lum_delay(18);
 END IF;

END PROCESS P1;

END inside;

