-------------------------------------------------------
-- Created by Ben Cope of Imperial College (April 2005)
-- Colour Balance Block for Primary Colour Correction
-------------------------------------------------------

LIBRARY IEEE;
USE IEEE.std_logic_1164.ALL;
USE IEEE.std_logic_arith.ALL;

LIBRARY work;
USE work.Constants.ALL;
USE work.ALL;

ENTITY Colour_Balance IS
PORT(
  -- Input Signals
  clock : IN STD_LOGIC;
  reset: IN STD_LOGIC; 
  RGB_in : IN VectorRGB_1x3;
  lum_shift : IN Vector12;
  d_rotate_vector : IN Vector18;
  negate_angle : IN STD_LOGIC;
  sat_shift: IN Vector18;
  area_selection: IN Vector3;
  TH1: IN Vector18;
  TH2: IN Vector18; 

  -- Output Signals
  RGB_out : OUT VectorRGB_1x3
);
END Colour_Balance;

ARCHITECTURE inside OF Colour_Balance IS

-- Internal Signals
SIGNAL XYL, XYL_pipelined: Vector18_1x3;
SIGNAL XYL_new, XYL_new_pipelined : Vector18_1x3;
SIGNAL RGB_edited, RGB, RGB_out_1, RGB_out_2, RGB_out_3 : VectorRGB_1x3;
SIGNAL RGB_delayed : Holding_RGB_Array;
SIGNAL Lum_delayed : Holding_Lum_Array;

BEGIN

-- Convert to XYL Space
C1: ENTITY rgb_to_xyl PORT MAP(

clock => clock,
reset => reset,
RGB => RGB_in,
XYL => XYL);

-- Edit Luminance
L1: ENTITY CB_Lum_alter PORT MAP(

clock => clock,
reset => reset,
lum_shift => lum_shift, 
lum_in => XYL_pipelined(2),
lum_out => XYL_new(2));

-- Edit CbCr
cbcr1: ENTITY CB_cbcr_alter PORT MAP(

clock => clock,
reset => reset,
d_rotate_vector => d_rotate_vector,
negate_angle => negate_angle,
sat_shift => sat_shift,
cb_in => XYL_pipelined(0),
cr_in => XYL_pipelined(1),
cb_out => XYL_new(0),
cr_out => XYL_new(1));

-- Convert back to RGB space
C2: ENTITY xyl_to_rgb PORT MAP(

clock => clock,
reset => reset,
XYL => XYL_new_pipelined,
RGB => RGB);

P1: PROCESS 
BEGIN
 WAIT UNTIL Rising_Edge(clock);
  XYL_pipelined <= XYL;
  XYL_new_pipelined <= XYL_new;
  RGB_edited <= RGB;
END PROCESS P1;

delay_lum: PROCESS 
BEGIN
  WAIT UNTIL Rising_Edge(clock);
   
	-- Keep version of Luminance
	lum_delayed(0) <= XYL(2);
	FOR i IN 0 TO xyl_rgb_latency + cordic_latency+2 LOOP
	 lum_delayed(i+1) <= lum_delayed(i);
	END LOOP;

	-- Keep version of RGB
	RGB_delayed(0) <= RGB_in;
	FOR i IN 0 TO total_latency_cb-1 LOOP
	 RGB_delayed(i+1) <= RGB_delayed(i);
	END LOOP;  

	-- Options
	IF (signed(lum_delayed(xyl_rgb_latency + cordic_latency)) >= signed(TH2)) THEN
	    RGB_out_1 <= RGB_edited;
	  ELSE
	   RGB_out_1 <= RGB_delayed(total_latency_cb-1);
   END IF;

   IF ((signed(lum_delayed(xyl_rgb_latency + cordic_latency)) < signed(TH2)) AND  
	     (signed(lum_delayed(xyl_rgb_latency + cordic_latency)) > signed(TH1))) THEN
	  RGB_out_2 <= RGB_edited;
	ELSE
	  RGB_out_2 <= RGB_delayed(total_latency_cb-1);
	END IF;
	
	IF (signed(lum_delayed(xyl_rgb_latency + cordic_latency)) <= signed(TH1)) THEN
    RGB_out_3 <= RGB_edited;
   ELSE
    RGB_out_3 <= RGB_delayed(total_latency_cb-1);
   END IF;

	-- Decide between RGB original and edited
	IF (area_selection(0) = '1') THEN
	  RGB_out <= RGB_out_1;
	ELSIF (area_selection(1) = '1') THEN
	  RGB_out <= RGB_out_2;
	ELSIF (area_selection(2) = '1') THEN
	  RGB_out <= RGB_out_3;
	ELSE
	   RGB_out <= RGB_delayed(total_latency_cb);
	END IF;
	 
END PROCESS delay_lum;

END inside;