-------------------------------------------------------
-- Created by Ben Cope of Imperial College (May 2005)
-- Cordic Processor for Primary Colour Correction
-------------------------------------------------------

-- Main Cordic Processor

LIBRARY IEEE;
USE IEEE.std_logic_1164.ALL;
USE IEEE.std_logic_arith.ALL;

LIBRARY work;
USE work.Constants.ALL;
USE work.ALL;

ENTITY Cordic_rotate IS
 PORT(
  -- Input Signals 
  clock : IN STD_LOGIC;
  reset : IN STD_LOGIC;
  negate : IN STD_LOGIC;
  d : IN Vector18;
  x_in : IN Vector19;
  y_in : IN Vector19;

  -- Output Signals
  x_out : OUT Vector19;
  y_out : OUT Vector19);
END Cordic_rotate;

ARCHITECTURE inside OF Cordic_rotate IS
 SIGNAL x,y : Cordic_Array;
 CONSTANT one : Vector19 := "0000000000000000001";
BEGIN

 Neg_proc: PROCESS
 VARIABLE rounding_x, rounding_y, sh_x, sh_y, x_n, y_n : Vector19;

 BEGIN
  WAIT UNTIL Rising_Edge(clock);
	IF (reset = '1') THEN
	 x_out <= (OTHERS => '0');
	 y_out <= (OTHERS => '0');
   ELSE 
	 IF (negate = '1') THEN
	  x_n := signed(not(x_in)) + signed(one);
	  y_n := signed(not(y_in)) + signed(one);
	  x(0) <= x_n(18 DOWNTO 0) & "00000000000000000";
	  y(0) <= y_n(18 DOWNTO 0) & "00000000000000000";
	 ELSE
	  x(0) <= x_in(18 DOWNTO 0) & "00000000000000000";
	  y(0) <= y_in(18 DOWNTO 0) & "00000000000000000"; 
	 END IF;
	 rounding_x := "000000000000000000" & x(18)(16);
	 rounding_y := "000000000000000000" & y(18)(16);
	 sh_x := x(18)(35 DOWNTO 17);
	 sh_y := y(18)(35 DOWNTO 17);
	 x_out <= signed(sh_x) + signed(rounding_x);
    y_out <= signed(sh_y) + signed(rounding_y);  
	END IF;

 END PROCESS Neg_proc;

G1: FOR i IN 0 TO 17 GENERATE
 Cordic1: ENTITY Cordic_block
  GENERIC MAP(
   n => i)
  PORT MAP(
   clock => clock,
	d => d(17-i),
--	n => i,
	x_in => x(i),
	y_in => y(i),
	x_out => x(i+1),
	y_out => y(i+1));
 END GENERATE;

END inside;