-------------------------------------------------------
-- Created by Ben Cope of Imperial College (April 2005)
-- Input Correction Block for Primary Colour Correction
-------------------------------------------------------

LIBRARY IEEE;
USE IEEE.std_logic_1164.ALL;
USE IEEE.std_logic_arith.ALL;

LIBRARY UNISIM;
USE UNISIM.vcomponents.ALL;

LIBRARY work;
USE work.Constants.ALL;
USE work.ALL;

ENTITY Input_Correction IS
PORT(
  -- Input Signals
  clock : IN STD_LOGIC;
  reset : IN STD_LOGIC; 
  RGB_in : IN VectorRGB_1x3;
  lum_shift : IN Vector12;	  -- range -1 to 1
  lum_gain : IN Vector15; -- range +/-8 downto +/- 0.25
  d_rotate_vector : IN Vector18;
  negate_angle : IN STD_LOGIC;
  sat_gain: IN Vector18; 
    
  -- Output Signals
  RGB_out : OUT VectorRGB_1x3
);
END Input_Correction;

ARCHITECTURE inside OF Input_Correction IS

-- Internal Signals
SIGNAL XYL, XYL_pipelined : Vector18_1x3;
SIGNAL XYL_new, XYL_new_pipelined : Vector18_1x3;
													
BEGIN

-- Convert to XYL Space
C1: ENTITY rgb_to_xyl PORT MAP(

clock => clock,
reset => reset,
RGB => RGB_in,
XYL => XYL);

-- Edit Luminance
L1: ENTITY IC_Lum_alter PORT MAP(

clock => clock,
reset => reset,
lum_shift => lum_shift, 
lum_gain => lum_gain,
lum_in => XYL_pipelined(2),
lum_out => XYL_new(2));

-- Edit CbCr
cbcr1: ENTITY IC_cbcr_alter PORT MAP(

clock => clock,
reset => reset,
d_rotate_vector => d_rotate_vector,
negate_angle => negate_angle,
sat_gain => sat_gain,
cb_in => XYL_pipelined(0),
cr_in => XYL_pipelined(1),
cb_out => XYL_new(0),
cr_out => XYL_new(1));

-- Convert back to RGB space
C2: ENTITY xyl_to_rgb PORT MAP(

clock => clock,
reset => reset,
XYL => XYL_new_pipelined,
RGB => RGB_out);

P1: PROCESS
BEGIN
 WAIT UNTIL Rising_Edge(clock);
  XYL_pipelined <= XYL;
  XYL_new_pipelined <= XYL_new;
END PROCESS P1;


END inside;
