-------------------------------------------------------
-- Created by Ben Cope of Imperial College (April 2005)
-- Top Level for Primary Colour Correction
-------------------------------------------------------

LIBRARY IEEE;
USE IEEE.std_logic_1164.ALL;
USE IEEE.std_logic_arith.ALL;

LIBRARY work;
USE work.Constants.ALL;
USE work.ALL;

ENTITY Primary_ccr IS
PORT(
  -- Input Signals
  clock : IN STD_LOGIC;
  ram_clock: IN STD_LOGIC;
  reset: IN STD_LOGIC; 

  -- Input Correct Inputs
  RGB_in : IN VectorRGB_1x3;
  lum_shift_ic : IN Vector12;	  -- range -1 to 1
  lum_gain : IN Vector15; -- range +/-8 downto +/- 0.25
  d_rotate_vector_ic : IN Vector18;
  negate_angle_ic : IN STD_LOGIC;
  sat_gain: IN Vector18; 
  
  -- Histogram Equalisation Inputs
  black_level : IN Vector9;  -- format 8 0
  BW_DIFF : IN Vector18;	  -- format 17 +4
  equalise : IN Vector3;
  output_black_level : IN VectorRGB;
  output_white_level : IN VectorRGB;
  addr_update : IN VectorRGB;
  write_enable : IN STD_LOGIC;
  gamma_update : IN VectorRGB_1x3;

  -- Colour Balance Inputs
  lum_shift_cb : IN Vector12;
  d_rotate_vector_cb : IN Vector18;
  negate_angle_cb : IN STD_LOGIC;
  sat_shift: IN Vector18;
  area_selection: IN Vector3;
  TH1: IN Vector18;
  TH2: IN Vector18; 

  -- Output Signals
  RGB_out : OUT VectorRGB_1x3
);
END Primary_ccr;

ARCHITECTURE inside OF Primary_ccr IS

-- Internal Signals
SIGNAL RGB_1, RGB_1_pipelined : VectorRGB_1x3;
SIGNAL RGB_2, RGB_2_pipelined : VectorRGB_1x3;

BEGIN

IC1: ENTITY Input_Correction PORT MAP(

clock => clock,

reset => reset,
RGB_in => RGB_in,
lum_shift => lum_shift_ic,
lum_gain => lum_gain,
d_rotate_vector => d_rotate_vector_ic,
negate_angle => negate_angle_ic,
sat_gain => sat_gain,
RGB_out => RGB_1);

HE1: ENTITY Histogram_Equalisation PORT MAP(

clock => clock,
ram_clock => ram_clock,
reset => reset,
RGB_in => RGB_1_pipelined,
black_level => black_level,
BW_DIFF => BW_DIFF,
equalise => equalise,
output_black_level => output_black_level,
output_white_level => output_white_level,
addr_update => addr_update,
write_enable => write_enable,
gamma_update => gamma_update,
RGB_out => RGB_2);

CB1: ENTITY Colour_Balance PORT MAP(

clock => clock,
reset => reset,
RGB_in => RGB_2_pipelined,
lum_shift => lum_shift_cb,
d_rotate_vector => d_rotate_vector_cb,
negate_angle => negate_angle_cb,
sat_shift => sat_shift,
area_selection => area_selection,
TH1 => TH1,
TH2 => TH2,
RGB_out => RGB_out);

P1: PROCESS

BEGIN
 WAIT UNTIL Rising_Edge(clock);
  RGB_1_pipelined <= RGB_1;
  RGB_2_pipelined <= RGB_2;
END PROCESS P1;

END inside;