-------------------------------------------------------
-- Created by Ben Cope of Imperial College (April 2005)
-- XYL to RGB conversion for Primary Colour Correction
-------------------------------------------------------

LIBRARY IEEE;
USE IEEE.std_logic_1164.ALL;
USE IEEE.std_logic_arith.ALL;

LIBRARY UNISIM;
USE UNISIM.vcomponents.ALL;

LIBRARY work;
USE work.Constants.ALL;

ENTITY xyl_to_rgb IS
 PORT(
  -- Input Signals 
  clock : IN STD_LOGIC;
  reset : IN STD_LOGIC;
  XYL : IN Vector18_1x3;
  
  -- Output Signals
  RGB : OUT VectorRGB_1x3);
END xyl_to_rgb;
  
ARCHITECTURE inside OF xyl_to_rgb IS

SIGNAL result: Matrix36_3x3;
SIGNAL ycbcr: Vector18_1x3;
SIGNAL minterm0, minterm1, rgb_int : Vector12_1x3;
SIGNAL A: Vector18_1x3;
SIGNAL x: Matrix12_3x3;
SIGNAL rounding: Vector12_1x3;
BEGIN

--  -----------------------------------------------------------
--  -- Concurrent Assignments - Buffer XYL signals with zeros
--  -----------------------------------------------------------
  A(0)(17) <= ycbcr(0)(17);
  A(0)(16) <= ycbcr(0)(17);  -- sign extension
  A(0)(15 DOWNTO 0) <= ycbcr(0)(17 DOWNTO 2);
  A(1)(17) <= ycbcr(1)(17);
  A(1)(16) <= ycbcr(1)(17);  -- sign extension
  A(1)(15 DOWNTO 0) <= ycbcr(1)(17 DOWNTO 2);
  A(2)(17) <= ycbcr(2)(17);
  A(2)(16) <= ycbcr(2)(17);  -- sign extension
  A(2)(15 DOWNTO 0) <= ycbcr(2)(17 DOWNTO 2);
  
 ------------------------------------------------------------
 -- Multipliers for Matrix
 ------------------------------------------------------------
 -- ROW 0
 MULT18X18S_inst1 : ENTITY Multiplier_model
    PORT MAP (
     P => result(0)(0),    -- 36-bit multiplier output
     A => A(0),    -- XYL Channel
     B => ycbcr_rgb_mx(0)(0),    -- Constant Matrix
     C => clock,    -- Clock input
     CE => '1',  -- Clock enable input
     R => reset     -- Synchronous reset input
    );

 MULT18X18S_inst2 : ENTITY Multiplier_model
 PORT MAP (
  P => result(0)(2),    -- 36-bit multiplier output
  A => A(2),    -- XYL Channel
  B => ycbcr_rgb_mx(0)(2),    -- Constant Matrix
  C => clock,    -- Clock input
  CE => '1',  -- Clock enable input
  R => reset     -- Synchronous reset input
  );

 result(0)(1) <= (OTHERS => '0');

 -- ROW 1
 result(1)(0) <= result(0)(0);

 G1: FOR j IN 1 TO 2 GENERATE	  -- Loop through RGB channels
    MULT18X18S_inst : ENTITY Multiplier_model
    PORT MAP (
     P => result(1)(j),    -- 36-bit multiplier output
     A => A(j),    -- XYL Channel
     B => ycbcr_rgb_mx(1)(j),    -- Constant Matrix
     C => clock,    -- Clock input
     CE => '1',  -- Clock enable input
     R => reset     -- Synchronous reset input
    );
 END GENERATE;

 -- ROW 2
 result(2)(0) <= result(0)(0);
 
 MULT18X18S_inst3 : ENTITY Multiplier_model
 PORT MAP (
  P => result(2)(1),    -- 36-bit multiplier output
  A => A(1),    -- XYL Channel
  B => ycbcr_rgb_mx(2)(1),    -- Constant Matrix
  C => clock,    -- Clock input
  CE => '1',  -- Clock enable input
  R => reset     -- Synchronous reset input
 );
 
 result(2)(2) <= (OTHERS => '0');

 Main_proc : PROCESS

  VARIABLE a,b,c: STD_LOGIC_VECTOR(0 TO 2);
   
  BEGIN

   WAIT UNTIL Rising_Edge(clock);
   
   IF (reset = '1') THEN
	 
	 ycbcr <= zeros18_1x3;
    rgb_int <= zeros12_1x3;
    minterm0 <= zeros12_1x3;
	 minterm1 <= zeros12_1x3;

   ELSE
		  
 -------------------------------------------------------------
 -- Pre-process input
 ------------------------------------------------------------- 
 
	 ycbcr(0) <= signed(XYL(2)) + signed(ycbcr_rgb_vec(0));
	 ycbcr(1) <= XYL(0);
	 ycbcr(2) <= XYL(1);
    
 -------------------------------------------------------------
 -- Multiplication happens in next cycle (see above)
 ------------------------------------------------------------- 
  
 -------------------------------------------------------------
 -- Perform Additions on Martix Products - output named ycbcr
 -------------------------------------------------------------
	  -- Reduce to same # bits as input (no delay)
	  FOR i IN 0 TO 2 LOOP -- r g or b line
	   FOR j IN 0 TO 2 LOOP	
		 x(i)(j) <= result(i)(j)(32 DOWNTO 21);  -- 3 integer bits
      END LOOP;
		a(i) := result(i)(0)(20);
		b(i) := result(i)(1)(20);
		c(i) := result(i)(2)(20);
		rounding(i) <= "0000000000" & ((a(i) and b(i)) or (b(i) and c(i)) or (c(i) and a(i))) & (a(i) xor b(i) xor c(i));
	  END LOOP;

	  -- PIPELINED ADDITION
	   minterm0(0) <= x(0)(0);
		minterm0(1) <= signed(x(1)(0)) + signed(x(1)(1)); 
		minterm0(2) <= signed(x(2)(0)) + signed(x(2)(1));

		minterm1(0) <= signed(x(0)(2)) + signed(rounding(0));
		minterm1(1) <= signed(x(1)(2)) + signed(rounding(1));
		minterm1(2) <= rounding(2);

	  FOR i IN 0 TO 2 LOOP
	   rgb_int(i) <= signed(minterm0(i)) + signed(minterm1(i));
     END LOOP;

	 END IF;

 END PROCESS Main_proc;

  -------------------------------------------------------------
 -- Range Limit
 ------------------------------------------------------------- 
 Range_Limit : PROCESS   
  BEGIN

   WAIT UNTIL Rising_Edge(clock);
   
   IF (reset = '1') THEN
    RGB <= zerosRGB_1x3;
   ELSE
		  
    FOR i IN 0 TO 2 LOOP

	  IF (signed(rgb_int(i)) < signed(zeros12_1x3(i))) THEN
	   RGB(i) <= RGB_zeros;
     ELSIF (signed(rgb_int(i)) > signed(one_12)) THEN
	   RGB(i) <= RGB_ones;
     ELSE 
	   RGB(i) <= rgb_int(i)(8 DOWNTO 1); 
     END IF;

	 END LOOP;
    
    END IF;
    
  END PROCESS Range_Limit;


END inside;