#include <math.h>
#include <iostream>
#ifdef WIN32
# include <windows.h>
#endif
#ifdef __APPLE__
#include <OpenGL/gl.h>
#include <OpenGL/glext.h>
#include <GLUT/glut.h>
#else
# include <GL/gl.h>
# include <GL/glut.h>
# include <GL/glext.h>
#endif
#include <Cg\cg.h>
#include <Cg\cgGL.h>

#include "CC_params.h"

#ifndef CWD
# define CWD ""
#endif

static CGcontext Context = NULL;
static CGprogram fragmentProgram = NULL;
static CGprogram vertexProgram = NULL;

//----------------------------------------------------
// Input uniform parameters (primary ccr)
//----------------------------------------------------
// Input Correct Parameters
static CGparameter B_lumShiftParam = NULL;
static CGparameter W_lumGainParam = NULL;
static CGparameter SatGainParam = NULL;
static CGparameter HueParam = NULL;

// Histogram Connect Parameters
static CGparameter ChannelSelectionParam = NULL;
static CGparameter BlackLevelParam = NULL;
static CGparameter GammaParam = NULL;
static CGparameter Gamma_preParam = NULL;
static CGparameter BW_DIFFParam = NULL;
static CGparameter OutputBlackLevelParam = NULL;
static CGparameter OutputWhiteLevelParam = NULL;

// Color Balance Parameters
static CGparameter AreaSelectionParam = NULL;
//static CGparameter HSLShiftParam = NULL;
static CGparameter HueShiftParam = NULL;
static CGparameter SatShiftParam = NULL;
static CGparameter LumShiftParam = NULL;
static CGparameter TH1Param = NULL;
static CGparameter TH2Param = NULL;

//----------------------------------------------------
// Input uniform parameters (secondary ccr)
//----------------------------------------------------

static CGparameter centerHueParam = NULL;
static CGparameter hueRangeParam = NULL;
static CGparameter hueSoftnessParam = NULL;
static CGparameter satMinParam = NULL;
static CGparameter satMaxParam = NULL;
static CGparameter satSoftnessParam = NULL;
static CGparameter lumMaxParam = NULL;
static CGparameter lumMinParam = NULL;
static CGparameter lumSoftnessParam = NULL;
static CGparameter deltaHueParam = NULL;
static CGparameter deltaSatParam = NULL;
static CGparameter deltaLumParam = NULL;

static CGparameter satSoftnessTypeParam = NULL;
static CGparameter lumSoftnessTypeParam = NULL;
static CGparameter hueSoftnessTypeParam = NULL;

static CGparameter keyInvertParam = NULL;

static CGparameter keyEdgeShiftLeftParam = NULL;
static CGparameter keyEdgeShiftRightParam = NULL;

// Vertex Program Parameters
static CGparameter ShiftParam = NULL;
static CGparameter ScaleParam = NULL;
static CGparameter RotateParam = NULL;

//----------------------------------------------------
// Non-uniform parameters 
//----------------------------------------------------

static CGparameter testTextureParam = NULL;
static CGparameter decalCoordsParam = NULL;

static CGprofile fragmentprofile;
static CGprofile vertexprofile;

//----------------------------------------------------
// Input uniform parameters (primary ccr)
//----------------------------------------------------
// Input Correct Parameters
static GLfloat Black = 0.0; 
static GLfloat White = 0.0;
static GLfloat lumGain = 1.0;
static GLfloat Saturation = 100.0;
static GLfloat SatGain = 0.0;
static GLfloat Hue = 0.0; 

// Histogram Correct Parameters
static GLfloat ChannelSelection = 0.5;
static GLfloat BlackLevel = 0.0;
static GLfloat Gamma = 0.0;
static GLfloat Gamma_pre = 0.0;
static GLfloat WhiteLevel = -1.0;	
static GLfloat BW_DIFF = 0.0;
static GLfloat OutputBlackLevel = 0.0;
static GLfloat OutputWhiteLevel = 1.0;

// Color Balance Parameters
static GLfloat AreaSelection = 1.0;
static GLfloat HueShift = 0.0;
static GLfloat SatShift = 0.0;
static GLfloat LumShift = 0.0;
static GLfloat HSLShift[3] = {0.0, 0.0, 0.0};
static GLfloat HighlightMidtoneCross = 0.0;
static GLfloat MidtoneHighlightCross = 0.0;

//----------------------------------------------------
// Input uniform parameters (secondary ccr)
//----------------------------------------------------
static GLfloat centerHue = 180.0;
static GLfloat hueRange = 360.0;
static GLfloat hueSoftness = 0.0;
static GLfloat satMin = 90.0;
static GLfloat satMax = 110.0;
static GLfloat satSoftness = 0.0;
static GLfloat lumMax = 1.0;
static GLfloat lumMin = 0.0;
static GLfloat lumSoftness = 0.0;
static GLfloat deltaHue = 0.1;
static GLfloat deltaSat = 0.1;
static GLfloat deltaLum = 0.1;

static GLfloat satSoftnessType = 1.0;
static GLfloat lumSoftnessType = 1.0;
static GLfloat hueSoftnessType = 1.0;

static GLfloat keyInvert = 0.5;

static GLfloat keyEdgeShiftLeft = 0.0;
static GLfloat keyEdgeShiftRight = 0.0;

static GLfloat Scale[2] = {1.0, 1.0}; 
static GLfloat Shift[2] = {0.0, 0.0};
static GLfloat Rotate = 0.0;

// Number of points in the image
int xsize = 30;
int ysize = 30;

static GLfloat *uv;

bool load = true; 
bool image = true;

static const float PI = 3.1415926;