// Code for implementing primary or secondary colour correction

#include "CC_header.h"

bool primary = true;

static void CC_choseprofiles(void)
{
	if (cgGLIsProfileSupported(CG_PROFILE_FP30))  // Advanced Fragment Profile 
		 fragmentprofile = CG_PROFILE_FP30;
	 else if (cgGLIsProfileSupported(CG_PROFILE_ARBFP1))  // Standard Frag Profile
		 fragmentprofile = CG_PROFILE_ARBFP1;
	 else
	 {
		 fprintf(stderr, "Video card does not support fragment programs, exiting...\n");
		 exit(-1);
	 }
	 if (cgGLIsProfileSupported(CG_PROFILE_VP30))  // Advanced Vertex Profile 
		 vertexprofile = CG_PROFILE_VP30;
	 else if (cgGLIsProfileSupported(CG_PROFILE_ARBVP1))  // Standard Vertex Profile
		 vertexprofile = CG_PROFILE_ARBVP1;
	 else
	 {
		 fprintf(stderr, "Video card does not support vertex programs, exiting...\n");
		 exit(-1);
	 }
}
// Functions for Primary CCR
static void CC_loadparams_primary(void)
{
 // Assign to uniform inputs in the cg file

 // Input Correct
 B_lumShiftParam = cgGetNamedParameter(fragmentProgram, "B_lumShift");
 CheckCgError();
 W_lumGainParam = cgGetNamedParameter(fragmentProgram, "W_lumGain");
 CheckCgError();
 SatGainParam = cgGetNamedParameter(fragmentProgram, "SatGain");
 CheckCgError();
 HueParam = cgGetNamedParameter(fragmentProgram, "Hue");
 CheckCgError();

 // Histogram Correct
 ChannelSelectionParam = cgGetNamedParameter(fragmentProgram, "ChannelSelection");
 CheckCgError();
 BlackLevelParam = cgGetNamedParameter(fragmentProgram, "BlackLevel");
 CheckCgError();
 GammaParam = cgGetNamedParameter(fragmentProgram, "Gamma");
 CheckCgError();
 Gamma_preParam = cgGetNamedParameter(fragmentProgram, "Gamma_pre");
 CheckCgError();
 BW_DIFFParam = cgGetNamedParameter(fragmentProgram, "BW_DIFF");
 CheckCgError();
 OutputBlackLevelParam = cgGetNamedParameter(fragmentProgram, "OutputBlackLevel");
 CheckCgError();
 OutputWhiteLevelParam = cgGetNamedParameter(fragmentProgram, "OutputWhiteLevel");
 CheckCgError();

 // Color Balance
 AreaSelectionParam = cgGetNamedParameter(fragmentProgram, "AreaSelection");
 CheckCgError();
 //HSLShiftParam = cgGetNamedParameter(fragmentProgram, "HSLShift");
 //CheckCgError();
 HueShiftParam = cgGetNamedParameter(fragmentProgram, "HueShift");
 CheckCgError();
 SatShiftParam = cgGetNamedParameter(fragmentProgram, "SatShift");
 CheckCgError();
 LumShiftParam = cgGetNamedParameter(fragmentProgram, "LumShift");
 CheckCgError();
 TH1Param = cgGetNamedParameter(fragmentProgram, "TH1");
 CheckCgError();
 TH2Param = cgGetNamedParameter(fragmentProgram, "TH2");
 CheckCgError();

 // Non-uniform inputs in file
 testTextureParam = cgGetNamedParameter(fragmentProgram, "testTexture");
 CheckCgError();
 decalCoordsParam = cgGetNamedParameter(fragmentProgram, "IN.decalCoords");
 CheckCgError();

 // Vertex Parameters
 ShiftParam = cgGetNamedParameter(vertexProgram, "Shift");
 CheckCgError();
 ScaleParam = cgGetNamedParameter(vertexProgram, "Scale");
 CheckCgError();
 RotateParam = cgGetNamedParameter(vertexProgram, "Rotate");
 CheckCgError();
}

static void CC_assignparams_primary(void)
{
 //----------------------------------------------------
 // Set uniform parameters
 //----------------------------------------------------
 // Input Correct Parameters
 if (White == 0.0)
 {	 
   lumGain = 1.0;
 }
 else
 {
   if (White < 100)
   {
	 lumGain = (White / 100.0) * 0.5f + 0.5f;
   }
   else
   {
 	 lumGain = (White / 100.0);
   }
 }
 if(B_lumShiftParam != NULL)
	cgGLSetParameter1f(B_lumShiftParam, (235-16)*Black/200 - (lumGain-1)*16);
 if(W_lumGainParam != NULL)
	cgGLSetParameter1f(W_lumGainParam, lumGain);

if(SatGainParam != NULL)
{
	GLfloat Sat;
	if (Saturation <= 0)
	{
		Sat = 255;
		SatGain = 0.1;
	}	
	else
	{
        SatGain = Saturation/100;
		Sat = 1/SatGain;
	}
	cgGLSetParameter1f(SatGainParam, Sat);
}
if(HueParam != NULL)
{
    GLfloat *Hue_array;
	Hue_array = new GLfloat[4];
	Hue_array[0] = SatGain*cos(Hue*PI/180);
	Hue_array[1] = -SatGain*sin(Hue*PI/180);
	Hue_array[2] = SatGain*sin(Hue*PI/180);
	Hue_array[3] = SatGain*cos(Hue*PI/180);
 	cgSetMatrixParameterfr(HueParam, Hue_array);
	delete Hue_array;
}
 // Histogram Correct Parameters
 if(ChannelSelectionParam != NULL)
	cgGLSetParameter1f(ChannelSelectionParam, ChannelSelection);
 if(BlackLevelParam != NULL)
 	cgGLSetParameter1f(BlackLevelParam, BlackLevel);
 if(GammaParam != NULL)
	cgGLSetParameter1f(GammaParam, Gamma);
 if(Gamma_preParam != NULL)
 {
    if( Gamma < 0.0f )
			Gamma_pre = (Gamma + 1.0f) * 0.9f + 0.1f;
		else
			Gamma_pre = Gamma * 0.9f + 1.0f;
	Gamma_pre = 1/Gamma_pre;
	if (Gamma == 0.0)
		Gamma_pre = 1;
	cgGLSetParameter1f(Gamma_preParam, Gamma_pre);
 }
 if(BW_DIFFParam != NULL)
 {
    BW_DIFF = 1 / abs(WhiteLevel - BlackLevel);
 	cgGLSetParameter1f(BW_DIFFParam, BW_DIFF);
 }
 if(OutputBlackLevelParam != NULL)
 {
	 float lower;
	 if (Gamma != 0)
		 lower = max(OutputBlackLevel, 0.0);
	 else
		 lower = OutputBlackLevel;
	cgGLSetParameter1f(OutputBlackLevelParam, lower);
 }
 if(OutputWhiteLevelParam != NULL)
 {
	 float upper;
	 if (Gamma != 0)
		 upper = min(OutputWhiteLevel, 1.0);
	 else
		 upper = OutputWhiteLevel;
	cgGLSetParameter1f(OutputWhiteLevelParam, upper);
 }
	
 // Color Balance Parameters
 if(AreaSelectionParam != NULL)
	cgGLSetParameter1f(AreaSelectionParam, AreaSelection);
 if(HueShiftParam != NULL)
 {
	 GLfloat *HueShift_array;
	 HueShift_array = new GLfloat[4];
	 HueShift_array[0] = cos(HueShift*PI/180);
	 HueShift_array[1] = -sin(HueShift*PI/180);
	 HueShift_array[2] = sin(HueShift*PI/180);
	 HueShift_array[3] = cos(HueShift*PI/180);
 	 cgSetMatrixParameterfr(HueShiftParam, HueShift_array);
	 delete HueShift_array;
 }
 if(SatShiftParam != NULL)
	cgGLSetParameter1f(SatShiftParam, SatShift);
 if(LumShiftParam != NULL)
	cgGLSetParameter1f(LumShiftParam, LumShift*(235-16));
 if(TH1Param != NULL)
	cgGLSetParameter1f(TH1Param, (235-16)*MidtoneHighlightCross/255.0);
 if(TH2Param != NULL)
	cgGLSetParameter1f(TH2Param, (235-16)*HighlightMidtoneCross/255.0);

 // Vertex Parameters
 if(ShiftParam != NULL)
	cgGLSetParameter2f(ShiftParam, Shift[0], Shift[1]);
 if(ScaleParam != NULL)
	cgGLSetParameter2f(ScaleParam, Scale[0], Scale[1]);
 if(RotateParam != NULL)
	cgGLSetParameter1f(RotateParam, Rotate);
}
 void keyboard_primary(unsigned char key, int x, int y)
 {
	 switch(key)
	 {
	 // Vertex Program Variables
	 case 'x': Shift[0] += 0.1; break;
	 case 'X': Shift[0] -= 0.1; break;
	 case 'y': Shift[1] += 0.1; break;
	 case 'Y': Shift[1] -= 0.1; break;
	 case 'c': Scale[0] += 0.1; break;
	 case 'C': Scale[0] -= 0.1; break;
	 case 'u': Scale[1] += 0.1; break;
	 case 'U': Scale[1] -= 0.1; break;
 	 case 'r': Rotate += 0.1; break;
	 case 'R': Rotate -= 0.1; break;
	 case '9': load = true; image = !image; break; // refresh and swap lenna / face

	 // Fragment Program Variables
	 // Input Correct 
	 case 'b': Black += 10; break;
	 case 'B': Black -= 10; break;
	 case 'w': White += 10; break;
	 case 'W': White -= 10; break;
	 case 's': Saturation += 10; break;  // ???? Scale
	 case 'S': Saturation -= 10; break;
	 case 'h': Hue += 1; break;
	 case 'H': Hue -= 1; break;

	 // Histogram Equalise
	 case 'v': BlackLevel += 0.1; break;
	 case 'V': BlackLevel -= 0.1; break;
	 case 'g': Gamma += 0.1; break;
	 case 'G': Gamma -= 0.1; break;
 	 case 'q': WhiteLevel += 0.1; break;
	 case 'Q': WhiteLevel -= 0.1; break;
	 case 'n': OutputBlackLevel += 0.1; break;
	 case 'N': OutputBlackLevel -= 0.1; break;
 	 case 'e': OutputWhiteLevel += 0.1; break;
	 case 'E': OutputWhiteLevel -= 0.1; break;
	 case 'j': ChannelSelection += 1; break;
	 case 'J': ChannelSelection -= 1; break;

	 // Color Balance
	 case 'p': HueShift += 10; break;
	 case 'P': HueShift -= 10; break;
	 case '2': SatShift += 0.1; break;
	 case '"': SatShift -= 0.1; break;
 	 case '3': LumShift += 0.1; break;
	 case '': LumShift -= 0.1; break;
	 case '4': HighlightMidtoneCross += 1; break;
	 case '$': HighlightMidtoneCross -= 1; break;
 	 case '5': MidtoneHighlightCross += 1; break;
	 case '%': MidtoneHighlightCross -= 1; break;
	 case '6': AreaSelection += 1; break;
	 case '^': AreaSelection -= 1; break;
	 }

	 glutPostRedisplay();
 }
// Functions for Secondary CCR
static void CC_loadparams_secondary(void)
{
 // Assign to uniform inputs in the cg file

 centerHueParam = cgGetNamedParameter(fragmentProgram, "centerHue");
 CheckCgError();
 hueRangeParam = cgGetNamedParameter(fragmentProgram, "hueRange");
 CheckCgError();
 hueSoftnessParam = cgGetNamedParameter(fragmentProgram, "hueSoftness");
 CheckCgError();
 satMinParam = cgGetNamedParameter(fragmentProgram, "satMin");
 CheckCgError();
 satMaxParam = cgGetNamedParameter(fragmentProgram, "satMax");
 CheckCgError();
 satSoftnessParam = cgGetNamedParameter(fragmentProgram, "satSoftness");
 CheckCgError();
 lumMaxParam = cgGetNamedParameter(fragmentProgram, "lumMax");
 CheckCgError();
 lumMinParam = cgGetNamedParameter(fragmentProgram, "lumMin");
 CheckCgError();
 lumSoftnessParam = cgGetNamedParameter(fragmentProgram, "lumSoftness");
 CheckCgError();
 deltaHueParam = cgGetNamedParameter(fragmentProgram, "deltaHue");
 CheckCgError();
 deltaSatParam = cgGetNamedParameter(fragmentProgram, "deltaSat");
 CheckCgError();
 deltaLumParam = cgGetNamedParameter(fragmentProgram, "deltaLum");
 CheckCgError();

 satSoftnessParam = cgGetNamedParameter(fragmentProgram, "satSoftness");
 CheckCgError();
 lumSoftnessParam = cgGetNamedParameter(fragmentProgram, "lumSoftness");
 CheckCgError();
 hueSoftnessParam = cgGetNamedParameter(fragmentProgram, "hueSoftness");
 CheckCgError();

 keyInvertParam = cgGetNamedParameter(fragmentProgram, "keyInvert");
 CheckCgError();
 keyEdgeShiftLeftParam = cgGetNamedParameter(fragmentProgram, "keyEdgeShiftLeft");
 CheckCgError();
 keyEdgeShiftRightParam = cgGetNamedParameter(fragmentProgram, "keyEdgeShiftRight");
 CheckCgError();

 // Non-uniform inputs in file
 testTextureParam = cgGetNamedParameter(fragmentProgram, "testTexture");
 CheckCgError();
 decalCoordsParam = cgGetNamedParameter(fragmentProgram, "IN.decalCoords");
 CheckCgError();

 // Vertex Parameters
 ShiftParam = cgGetNamedParameter(vertexProgram, "Shift");
 CheckCgError();
 ScaleParam = cgGetNamedParameter(vertexProgram, "Scale");
 CheckCgError();
 RotateParam = cgGetNamedParameter(vertexProgram, "Rotate");
 CheckCgError();
}

static void CC_assignparams_secondary(void)
{
 //----------------------------------------------------
 // Set uniform parameters
 //----------------------------------------------------

 if(centerHueParam != NULL)
	cgGLSetParameter1f(centerHueParam, centerHue);
 if(hueRangeParam != NULL)
	cgGLSetParameter1f(hueRangeParam, hueRange);
 if(hueSoftnessParam != NULL)
	cgGLSetParameter1f(hueSoftnessParam, hueSoftness);
 if(satMinParam != NULL)
	cgGLSetParameter1f(satMinParam, satMin);
 if(satMaxParam != NULL)
	cgGLSetParameter1f(satMaxParam, satMax);
 if(satSoftnessParam != NULL)
 	cgGLSetParameter1f(satSoftnessParam, satSoftness);
 if(lumMaxParam != NULL)
	cgGLSetParameter1f(lumMaxParam, lumMax);
 if(lumMinParam != NULL)
 	cgGLSetParameter1f(lumMinParam, lumMin);
 if(lumSoftnessParam != NULL)
	cgGLSetParameter1f(lumSoftnessParam, lumSoftness);
 if(deltaHueParam != NULL)
	cgGLSetParameter1f(deltaHueParam, deltaHue);
 if(deltaSatParam != NULL)
	cgGLSetParameter1f(deltaSatParam, deltaSat);
 if(deltaLumParam != NULL)
	cgGLSetParameter1f(deltaLumParam, deltaLum);
 
 if(satSoftnessTypeParam != NULL)
 	cgGLSetParameter1f(satSoftnessTypeParam, satSoftnessType);
 if(lumSoftnessTypeParam != NULL)
	cgGLSetParameter1f(lumSoftnessTypeParam, lumSoftnessType);
 if(hueSoftnessTypeParam != NULL)
	cgGLSetParameter1f(hueSoftnessTypeParam, hueSoftnessType);
 
 if(keyInvertParam != NULL)
	cgGLSetParameter1f(keyInvertParam, keyInvert);
 
 if(keyEdgeShiftLeftParam != NULL)
	cgGLSetParameter1f(keyEdgeShiftLeftParam, keyEdgeShiftLeft);
 if(keyEdgeShiftRightParam != NULL)
	cgGLSetParameter1f(keyEdgeShiftRightParam, keyEdgeShiftRight);

 // Vertex Parameters
 if(ShiftParam != NULL)
	cgGLSetParameter2f(ShiftParam, Shift[0], Shift[1]);
 if(ScaleParam != NULL)
	cgGLSetParameter2f(ScaleParam, Scale[0], Scale[1]);
 if(RotateParam != NULL)
	cgGLSetParameter1f(RotateParam, Rotate);
}

 void keyboard_secondary(unsigned char key, int x, int y)
 {
	 switch(key)
	 {
	 // Vertex Program Variables
	 case 'x': Shift[0] += 0.1; break;
	 case 'X': Shift[0] -= 0.1; break;
	 case 'y': Shift[1] += 0.1; break;
	 case 'Y': Shift[1] -= 0.1; break;
	 case 'c': Scale[0] += 0.1; break;
	 case 'C': Scale[0] -= 0.1; break;
	 case 'u': Scale[1] += 0.1; break;
	 case 'U': Scale[1] -= 0.1; break;
 	 case 'r': Rotate += 0.1; break;
	 case 'R': Rotate -= 0.1; break;
	 case '9': load = true; image = !image; break; // refresh and swap lenna / face

	 // Fragment Program Variables
	 case '1': centerHue += 10; break;
	 case '!': centerHue -= 10; break;
	 case '2': hueRange += 10; break;
	 case '"': hueRange -= 10; break;
	 case '3': hueSoftness += 5; break;  // ???? Scale
	 case '': hueSoftness -= 5; break;
	 case '4': satMin += 10; break;
	 case '$': satMin -= 10; break;
	 case '5': satMax += 10; break;
	 case '%': satMax -= 10; break;
	 case '6': satSoftness += 0.1; break;
	 case '^': satSoftness -= 0.1; break;
 	 case '7': lumMax += 10; break;
	 case '&': lumMax -= 10; break;
	 case '8': lumMin += 10; break;
	 case '*': lumMin -= 10; break;
 	 case 'a': lumSoftness += 10; break;
	 case 'A': lumSoftness -= 10; break;
	 case 'b': deltaHue += 1; break;
	 case 'B': deltaHue -= 1; break;
	 case 'd': deltaSat += 10; break;
	 case 'D': deltaSat -= 10; break;
	 case 'e': deltaLum += 1; break;
	 case 'E': deltaLum -= 1; break;
 	 
	 case 'f': satSoftnessType += 10; break;
	 case 'F': satSoftnessType -= 10; break;
	 case 'g': lumSoftnessType += 10; break;
	 case 'G': lumSoftnessType -= 10; break;
 	 case 'h': hueSoftnessType += 10; break;
	 case 'H': hueSoftnessType -= 10; break;
	 
	 case 'i': keyInvert += 1; break;
	 case 'I': keyInvert -= 1; break;

	 case 'j': keyEdgeShiftLeft += 1; break;
	 case 'J': keyEdgeShiftLeft -= 1; break;
	 case 'k': keyEdgeShiftRight += 1; break;
	 case 'K': keyEdgeShiftRight -= 1; break;
	 }

	 glutPostRedisplay();
 }

// Image Displaying
 static void Display_image(void)
{
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	cgGLBindProgram(fragmentProgram);
	CheckCgError();

	cgGLBindProgram(vertexProgram);
	CheckCgError();
    if (primary)
        CC_assignparams_primary();
	else CC_assignparams_secondary();

	if (load)
	{
		//CC_loadtexture();
		// Choose which image to Load - Insert own...
		if (image)
		{
            if (load_texture("lennaodd.bmp"))
				printf("Texture Read Sucessful...\n");
		}
		else
		{
            if (load_texture("face.ppm"))
				printf("Texture Read Sucessful...\n");
		}

	 
		cgGLSetTextureParameter(testTextureParam, g_Texture.id);

		cgGLEnableTextureParameter(testTextureParam);
		load = false;
	}

	cgGLEnableProfile(fragmentprofile);
	cgGLEnableProfile(vertexprofile);

	glBegin(GL_QUADS);
		glVertex2f(-1, -1);
		glVertex2f(-1, 1);
		glVertex2f(1, 1);
		glVertex2f(1, -1);
	glEnd();

	cgGLDisableTextureParameter(testTextureParam);
	
	cgGLDisableProfile(fragmentprofile);
	cgGLDisableProfile(vertexprofile);

	glutSwapBuffers();
}

static void InitializeGlut(int *argc, char *argv[])
{
	//glEnable(GL_TEXTURE_2D);
	glutInit(argc, argv);
	glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGB);
	if (primary)
        glutCreateWindow("Primary Colour Correction");
	else glutCreateWindow("Secondary Colour Correction");
	glutDisplayFunc(Display_image);
}

 int main(int argc, char *argv[])
 {
	 InitializeGlut(&argc, argv);

	 CC_choseprofiles();

	 // Test Cg Context creation
	 Context = cgCreateContext();
	 CheckCgError();

	 // Test adding source text to context
	 if (primary)
	 {
     fragmentProgram = cgCreateProgramFromFile(Context,
                                    CG_SOURCE, CWD "ccr_fragment.cg",
                                    fragmentprofile,
                                    "main", NULL);
	 }
	 else
     {
     fragmentProgram = cgCreateProgramFromFile(Context,
                                    CG_SOURCE, CWD "ccr_fragment_sec.cg",
                                    fragmentprofile,
                                    "main", NULL);
	 }
	 CheckCgError();
	 
	 fprintf(stderr, "LAST LISTING----%s----\n", cgGetLastListing(Context));

	 fprintf(stderr, "---- PROGRAM BEGIN ----\n%s---- PROGRAM END ----\n",
        cgGetProgramString(fragmentProgram, CG_COMPILED_PROGRAM));

	 if (fragmentProgram != NULL)
	 {
		 cgGLLoadProgram(fragmentProgram);
		 CheckCgError();
	 }

	 vertexProgram = cgCreateProgramFromFile(Context,
                                    CG_SOURCE, CWD "ccr_vertex.cg",
                                    vertexprofile,
                                    "ccr_vertex", NULL);
	 CheckCgError();

 	 fprintf(stderr, "LAST LISTING----%s----\n", cgGetLastListing(Context));

	 fprintf(stderr, "---- PROGRAM BEGIN ----\n%s---- PROGRAM END ----\n",
        cgGetProgramString(vertexProgram, CG_COMPILED_PROGRAM));

	 if (vertexProgram != NULL)
	 {
		 cgGLLoadProgram(vertexProgram);
		 CheckCgError();
	 }

	 if (fragmentProgram != NULL && vertexProgram != NULL)
	 {
		 if (primary)
             CC_loadparams_primary();
		 else
             CC_loadparams_secondary();
	 }

	 if (primary)
         glutKeyboardFunc(keyboard_primary);
	 else
         glutKeyboardFunc(keyboard_secondary);

     glutMainLoop();

	 cgDestroyProgram(fragmentProgram);
	 cgDestroyProgram(vertexProgram);

	 cgDestroyContext(Context);

	 glDeleteTextures(1, &g_Texture.id);

	 return 0;
 }